function figuresMunsWerker(figdir)
    N   = 40;  % number of participants & ages
    lam = 0.2; % risk premium on stocks
    sig = 0.2; % stock volatility
    vgamma = [.5 1 4/3 2 4 8]'; % some levels of risk aversion
    mgamma = repmat(vgamma, 1, N); % CRRA utility
    vage = -N:-1; % years until pension age
    r   = 0.01;   % interest rate
    rho = 0.01;   % expected return by period
    h   = 1;      % contribution per period
    Np  = 100;    % number of datapoints in figures
    
    if ~exist('figdir','var')
        figdir = [];
    end
    if (isempty(figdir) || figdir(2)~=':')
        figdir = [pwd '\' figdir];
    end

    % expected F and H by age:
    vF   = h*((1+rho).^(1+vage-min(vage))-1)/rho;
    % same: vH = h*(1+r)*cumsum((1+r).^vage ,'reverse') - 1;
    vH   = h*(1-(1+r).^(vage+1))./r;
    vFH  = vF+vH;
    vFfrac = vF./vFH;

    valp = vFH./sum(vFH,2);
    vwFB = (lam/sig./mgamma)./vFfrac;
    vv   = vFH.^2./(valp.*mgamma);
    vv   = vv./sum(vv,2);
    par  = struct('valp',valp,'mgammaa',mgamma,'lam',lam,'sig',sig);
    wFB  = (vwFB*vF(:))./sum(vF,2);
    vw     = linspace(0,1,Np)';
    vwiopt = cell(1,length(vgamma));
    CEQSBopt = NaN(Np, size(mgamma,1) );
    for i=1:length(vgamma)
        par.gamma = mgamma(i,:);
        vwiopt{i} = vwFB(i,:) + vv(i,:)./vF.*( vw*sum(vF) - vwFB(i,:)*vF(:) ); % Prop 3
        [CEQSBopt(:,i), ~] = U( vwiopt{i}, par, vFfrac);
    end
    close all;
    set(groot,'defaultAxesTickLabelInterpreter','latex');  
    set(groot,'defaulttextinterpreter','latex');
    set(groot,'defaultLegendInterpreter','latex');
    
    % Figure 1
    figure;
    subplot(2,1,1)
    makeFigure( 0:N, [[0 vF]; [ (vH(1)+h)/(1+r) vH]], {'$F_i$','$H_i$'}, 'years until pension age', 'amount', ...
        false, [], linspace(0,N,5), fliplr(linspace(0,N,5)), [], [], false, [], [], false, 'eastoutside', false);

    subplot(2,1,2)
    makeFigure( 1:N, par.lam./(par.sig.*vFfrac.*vgamma), [], 'years until pension age', '$w_{i}^{\rm FB}$', ...
        false, [NaN 10], linspace(0,N,5), fliplr(linspace(0,N,5)), [], [], false, vgamma, '\gamma_i', false, 'eastoutside', false);
    saveas(gcf, [figdir '\Fig_FHw_age'], 'epsc')

    % Figure 2
    figure;
    subplot(2,1,1)
    wFB1    = (par.lam/par.sig./vFfrac*vF(:))./sum(vF,2);
    vgamma0 = linspace(1,10);
    makeFigure(vgamma0, wFB1./vgamma0, [], '$\tilde\gamma$','$w^{\rm FB}$', false, [0 NaN], [], [], [], [], false, [], [], false, 'northeast', false);
    subplot(2,1,2)    
    makeFigure(vw, 100*CEQSBopt, [], '$w$','${\rm CEQ}^{\rm SB}\,(\%)$', ...
        false, [], 0:.2:1, wFB, 'w^{\rm FB}', [], true, vgamma, '\gamma_i', false, 'southwest', false);
    saveas(gcf, [figdir '\FiggamwCEQ'], 'epsc')
    
    % now including annuity risk
    par.lamA     = 0.1;
    par.Dpension = 20/2;
    RTSsens      = 1; % linear with 0,6% gives best fit with DNB euro swap rates
    par.vsigA    = 0.006 * (-(1+vage)/2 + par.Dpension).^RTSsens;
    par.vD       = 0.006 * (-(1+vage)).^RTSsens /2;
    vaFB         = ((par.lamA-par.vsigA)./mgamma + par.vsigA)./vFfrac - par.vD.*(1./vFfrac-1);
    
    % Figure 3
    figure;
    subplot(2,1,1)
    plot(vage, [par.vD; par.vsigA])
    makeFigure( 0:N, [2*par.vsigA(1)-par.vsigA(2) par.vsigA ; 2*par.vD(1)-par.vD(2)  par.vD], {'$\sigma_{A_i}$','$D_i$'},'years until pension age','exposure to $Z_A$', ...
        false, [0 .2], linspace(0,N,5), fliplr(linspace(0,N,5)), [], .05:.05:.2, false, [],  [], false, 'eastoutside', false);
    subplot(2,1,2)
    makeFigure( 1:N, vaFB, [], 'years until pension age','$a_i^{\rm FB}$', false, [-.1 .4], linspace(0,N,5), fliplr(linspace(0,N,5)), [], -.1:.1:.4, false, vgamma, '\gamma_i', false,'eastoutside', false);
    saveas(gcf, [figdir '\Fig_age_sigmaD'], 'epsc')
    
    % Figure 4
    figure;
    vgamma0  = linspace(.2,10);
    vaFB     = ((par.lamA-par.vsigA)./vgamma0(:) + par.vsigA)./vFfrac - par.vD.*(1./vFfrac-1);
    makeFigure( vgamma0, vaFB(:,5:10:end), [], '$\gamma$', '$a_i^{\rm FB}$', false, [-.1 .4], 0:2:max(vgamma0), [], [], -.1:.1:.4, false, (N-5):-10:0, '\rm ~years~until~pension~age', true, [], false);
    hold on;
    aFB      = sum(vaFB .* vF, 2)./sum(vF);
    h        = makeFigure( vgamma0, aFB, [], '$\gamma$', '$a_i^{\rm FB}$', false, [-.1 .4], 0:2:max(vgamma0), [], [], -.1:.1:.4, false, NaN, [], false, [], false);
    set(h,'Color','k','LineWidth',2,'DisplayName','fund');
    saveas(gcf, [figdir '\Figa_age_FB'], 'epsc')
    
    % Figure 5
    figure;
    subplot(1,2,1)
    vaFB     = ((par.lamA-par.vsigA)./vgamma(:) + par.vsigA)./vFfrac - par.vD.*(1./vFfrac-1);
    va       = linspace(0, .4, Np)';
    vaiopt   = cell(1,length(vgamma));
    CEQ_SBa  = NaN(Np, length(vgamma) );
    for i=1:length(vgamma)
        par.gamma = mgamma(i,:);
        vaiopt{i} = vaFB(i,:) + vv(i,:)./vF.*( va*sum(vF) - vaFB(i,:)*vF(:) ); % Prop 4
        [CEQ_SBa(:,i), ~] = U2( vwFB(i,:), vaiopt{i}, par, vFfrac);
    end
    makeFigure(va, 100*CEQ_SBa, [], '$a$','${\rm{CEQ}}^{\rm SB}\,(\%)$', false, [], 0:.1:.4, [], [], [], false, vgamma, '\tilde\gamma', false,'southwest', false);
    subplot(1,2,2)
    vwiopt_avar = cell(1,length(vgamma));
    CEQ_SBw = NaN(Np, length(vgamma) );
    for i=1:length(vgamma)
        par.gamma = mgamma(i,:);
        vwiopt_avar{i} = vwFB(i,:) + vv(i,:)./vF.*( vw*sum(vF) - vwFB(i,:)*vF(:) ); % Prop 4
        [CEQ_SBw(:,i), ~] = U2( vwiopt_avar{i}, vaFB(i,:), par, vFfrac);
    end
    makeFigure(vw, 100*CEQ_SBw, [], '$w$','${\rm{CEQ}}^{\rm SB}\,(\%)$', false, [], 0:.2:1, [], [], [], false, vgamma, '\tilde\gamma', false, 'southwest', false);
    saveas(gcf, [figdir '\FigCEQa_w'], 'epsc')
    
    vrho  = linspace(-.75,.75,151);
    wFB   = NaN(length(vgamma), length(vrho));
    aFB   = NaN(length(vgamma), length(vrho));
    for i=1:length(vgamma)
        par.gamma = mgamma(i,:);
        vwFB      = 1./(1-vrho.^2) .* (par.lam  - vrho.*par.lamA)./(par.sig .* vgamma(i))./vFfrac(:);
        wFB(i,:)  = (vF*vwFB)./sum(vF);
        
        vanetFB  = 1./(1-vrho.^2) .* (par.lamA - vrho.*par.lam + (vrho.^2-1).*par.vsigA(:))./vgamma(i)./vFfrac(:);
        vaFB     = vanetFB + par.vsigA(:)./vFfrac(:) - par.vD(:).*(1./vFfrac(:)-1);
        aFB(i,:) = (vF*vaFB)./sum(vF);
    end
    % Figure 6
    figure;
    subplot(2,1,1)
    makeFigure(vrho, wFB, [], '$\rho$', '$w^{\rm FB}$', false, [], -.75:0.25:.75, [], [], [], false, vgamma, '\tilde\gamma', false, 'northeast', false);
    subplot(2,1,2)
    makeFigure(vrho, aFB, [], '$\rho$', '$a^{\rm FB}$', false, [], -.75:0.25:.75, [], [], [], false, vgamma, '\tilde\gamma', false, 'northeast', false);
    saveas(gcf, [figdir '\FigFBrho'], 'epsc')    
end

function h0 = makeFigure(x, y, clegend, xlab, ylab, blogy, yl, defXTickValues, specXTickValue, specXTickLabel, defYTickValues, bgammaTick, vgamma, lablegend, bnumfirstlegend, locLegend, bnewplot)
    if ~exist('bnewplot','var') && isempty(bnewplot)
        bnewplot = true;
    end
    if bnewplot
        figure;
    end
    if blogy
        h0 = semilogy(x, y);
    else
        h0 = plot(x, y);
    end
    xlabel(xlab)
    ylabel(ylab)
    if exist('clegend','var') && ~isempty(clegend)
        if isempty(locLegend)
            legend(clegend)
        else
            legend(clegend, 'Location',locLegend)
        end
    end
    if min(y(:))>-1e-8 && min(y(:))<0
        ylim = get(gca,'ylim');
        set(gca, 'ylim', [0 ylim(2)])
    end
    if exist('defXTickValues','var') && any([ defXTickValues(:); specXTickValue(:) ])
        [vXTick, loc] = sort([specXTickValue(:)' setdiff(defXTickValues, specXTickValue)]);
        xlim([min(defXTickValues) max(defXTickValues)])
        xticks(unique(vXTick))
        XTickLabel = num2cell( get(gca,'XTick') );
        for i =1:length(specXTickValue)
            if bgammaTick
                blegend = false;
                if mod(vgamma(i),1)~=0
                    for j=2:5
                        if mod(vgamma(i)*j,1)==0
                            XTickLabel(loc==i) = {['$' specXTickLabel '_{' lablegend '=' num2str(vgamma(i)*j) '/' num2str(j) '$']};
                            blegend         = true;
                        end
                    end
                end
                if ~blegend
                    XTickLabel(loc==i) = {['$' specXTickLabel '_{' lablegend '=' num2str(vgamma(i)) '}$']};
                end
            else
                XTickLabel(i) = {specXTickValue(i)};
            end
        end
        xticklabels(XTickLabel)
        if exist('vgamma','var')
            legendentry = cell(1,length(vgamma));
            for i=1:length(vgamma)
                blegend = false;
                if mod(vgamma(i),1)~=0
                    for j=2:5
                        if mod(vgamma(i)*j,1)==0 && ~blegend
                            legendentry{i}  = getLegendEntry(lablegend, [num2str(vgamma(i)*j) '/' num2str(j)], bnumfirstlegend);
                            blegend         = true;
                        end
                    end
                end
                if ~blegend
                    legendentry{i}  = getLegendEntry(lablegend, num2str(vgamma(i)), bnumfirstlegend);
                end
            end
            if length(vgamma)>1 && isempty(clegend)
                if isempty(locLegend)
                    legend(legendentry);
                else
                    legend(legendentry, 'Location',locLegend);
                end
            end
        end
    end
    if exist('defYTickValues','var') && ~isempty(defYTickValues)
        yticks(unique(defYTickValues))
    end
    yl0 = get(gca,'ylim');
    if ~isempty(yl) && any(~isnan(yl))
        yl0(~isnan(yl)) = yl(~isnan(yl));
        set(gca,'ylim',yl0);
    end
    if (yl0(1) < 0 && yl0(2)>0)
        hold on;
        h = plot( get(gca,'xlim'),[0 0],':k');
        set(h,'HandleVisibility','off');
    end
    box off
end

function legEntry = getLegendEntry(lablegend, snum, bnumfirstlegend)
    if bnumfirstlegend
        legEntry = ['$' snum lablegend '$'];
    else
        legEntry = ['$' lablegend '=' snum '$'];
    end
end

function [CEQ_SP,CEQind] = U2(w,a, par, vFfrac)
    CEQind = NaN([size(w,1) size(a)]);
    for i=1:size(w,1)
        for j=1:size(a,1)
            vanet = a(j,:) - par.vsigA./vFfrac + par.vD.*(1./vFfrac-1);
            CEQind(i,j,:) = (par.lam*par.sig*w(i,:) + par.lamA.*vanet - par.vsigA.*a(j,:) ).*vFfrac + par.vsigA.^2 ...
                            - .5*par.gamma .* ( (par.sig *w(i,:)).^2 + vanet.^2 ) .* vFfrac.^2;
        end
    end
    CEQ_SP = sum( permute(par.valp, [1 3 2]) .* CEQind,3);
end

function [U,CEQ] = U(w, par, vFfrac)
    CEQ = NaN(size(w));
    for i=1:size(w,1)
        CEQ(i,:) =  par.lam*par.sig*w(i,:).*vFfrac - .5*par.gamma .* (par.sig *w(i,:) .* vFfrac).^2;
    end
    U = CEQ * par.valp(:);
end